from typing import TYPE_CHECKING, Optional
from uuid import UUID

from myst.client import get_client
from myst.core.time.time_delta import TimeDelta
from myst.models.enums import DeployStatus
from myst.models.timing import TimeRangeBoundary
from myst.models.types import AxisLabels, CoordinateLabels, ItemOrSlice, Shape
from myst.openapi.api.projects.time_series import create_time_series
from myst.openapi.models.time_series_create import TimeSeriesCreate
from myst.openapi.models.time_series_get import TimeSeriesGet
from myst.resources.resource import ShareableResource

if TYPE_CHECKING:
    from myst.resources.time_series import TimeSeries


class Node(ShareableResource):
    """A node in a project graph.

    Attributes:
        project: identifier of the project to which this node belongs
        title: the title of this node
        description: a brief description of the node
        deploy_status: whether this node is new, deployed, or inactive
    """

    project: UUID
    title: str
    description: Optional[str] = None
    deploy_status: DeployStatus

    def create_time_series(
        self,
        title: str,
        sample_period: TimeDelta,
        cell_shape: Shape = (),
        coordinate_labels: CoordinateLabels = (),
        axis_labels: AxisLabels = (),
        output_index: int = 0,
        label_indexer: Optional[ItemOrSlice] = None,
        start_timing: Optional[TimeRangeBoundary] = None,
        end_timing: Optional[TimeRangeBoundary] = None,
        description: Optional[str] = None,
    ) -> "TimeSeries":
        """Creates a single-layer time series downstream from this node.

        This method creates a time series and by a single layer whose downstream node is the newly created time series
        and whose upstream node is this node. Useful when creating a trivial time series just to capture the output of
        this node.

        Args:
            title: the title of the time series to create
            sample_period: the frequency at which the data is sampled
            cell_shape: the shape of the data in each cell
            coordinate_labels: the labels for each coordinate of a cell
            axis_labels: the labels for each axis of a cell
            output_index: which time dataset, out of the sequence of this node's time datasets, to pass to the layer
            label_indexer: the slice of this node's output data to pass into the layer
            start_timing: the beginning of the natural time range the layer should produce data for; if None, there is
                no restriction on the beginning of the range
            end_timing: the end of the natural time range the layer should produce data for; if None, there is no
                restriction on the end of the range
            description: a brief description of the time series

        Returns:
            the newly created time series
        """
        # Avoid circular imports.
        from myst.resources.time_series import TimeSeries

        time_series_get: TimeSeriesGet = create_time_series.request_sync(
            client=get_client(),
            project_uuid=str(self.project),
            json_body=TimeSeriesCreate(
                object="Node",
                type="TimeSeries",
                title=title,
                sample_period=sample_period.to_iso_string(),
                cell_shape=cell_shape,
                coordinate_labels=coordinate_labels,
                axis_labels=axis_labels,
                description=description,
            ),
        )

        time_series = TimeSeries.parse_obj(time_series_get.dict())

        time_series.create_layer(
            node=self,
            order=0,
            output_index=output_index,
            label_indexer=label_indexer,
            start_timing=start_timing,
            end_timing=end_timing,
        )

        return time_series



from typing import TYPE_CHECKING, Any, Dict, List, Optional, Union

from myst.adapters.timing import to_timing_create
from myst.adapters.utils import drop_unset_values, get_resource_uuid
from myst.client import get_client
from myst.connectors.model_connector import ModelConnector
from myst.core.time.time import Time
from myst.core.time.time_delta import TimeDelta
from myst.models.timing import ScheduleTiming, TimeRangeBoundary, from_time_range_boundary
from myst.models.types import UNSET, ItemOrSlice, OptionalArgument, UUIDOrStr, to_uuid
from myst.openapi.api.projects.models import create_model, get_model, model_fit_job_create, update_model
from myst.openapi.models.model_create import ModelCreate
from myst.openapi.models.model_update import ModelUpdate
from myst.openapi.models.node_job_create import NodeJobCreate
from myst.resources.connector_node import ConnectorNode
from myst.resources.input import ModelInput
from myst.resources.job import ModelFitJob
from myst.resources.policy import ModelFitPolicy
from myst.resources.result import ModelFitResult, ModelRunResult, ModelRunResultMetadata
from myst.resources.time_series import TimeSeries

if TYPE_CHECKING:  # Avoid circular imports.
    from myst.resources.project import Project


class Model(ConnectorNode):
    """A node that learns its parameters during a training phase, and produces output during a prediction phase."""

    @classmethod
    def create(
        cls,
        project: Union["Project", UUIDOrStr],
        title: str,
        connector: ModelConnector,
        description: Optional[str] = None,
    ) -> "Model":
        """Creates a new model node.

        Args:
            project: the project in which to create the model
            title: the title of the model
            connector: the model connector to use in the model node
            description: a brief description of the model

        Returns:
            the newly created model
        """
        model = create_model.request_sync(
            client=get_client(),
            project_uuid=str(get_resource_uuid(project)),
            json_body=ModelCreate(
                object="Node",
                type="Model",
                title=title,
                description=description,
                connector_uuid=str(connector.uuid),
                parameters=connector.parameters_exclude_none(),
            ),
        )

        return Model.parse_obj(model.dict())

    @classmethod
    def get(cls, project: Union["Project", UUIDOrStr], uuid: UUIDOrStr) -> "Model":
        """Gets a specific model by its identifier."""
        model = get_model.request_sync(
            client=get_client(), project_uuid=str(get_resource_uuid(project)), uuid=str(to_uuid(uuid))
        )

        return Model.parse_obj(model.dict())

    @classmethod
    def list(cls, project: Union["Project", UUIDOrStr]) -> List["Model"]:
        """Gets all models by project."""
        raise NotImplementedError()

    def update(
        self,
        title: OptionalArgument[str] = UNSET,
        connector: OptionalArgument[ModelConnector] = UNSET,
        description: OptionalArgument[str] = UNSET,
    ) -> "Model":
        """Updates model."""
        parameters = connector.parameters_exclude_none() if isinstance(connector, ModelConnector) else UNSET

        model_update: Dict[str, Any] = dict(
            object="Node",
            type="Model",
            title=title,
            connector_uuid=str(connector.uuid) if isinstance(connector, ModelConnector) else UNSET,
            parameters=parameters,
            description=description,
        )

        model = update_model.request_sync(
            client=get_client(),
            project_uuid=str(self.project),
            uuid=str(self.uuid),
            json_body=ModelUpdate.parse_obj(drop_unset_values(model_update)),
        )

        return Model.parse_obj(model.dict())

    def create_input(
        self,
        time_series: Union[TimeSeries, UUIDOrStr],
        group_name: str,
        output_index: int = 0,
        label_indexer: Optional[ItemOrSlice] = None,
    ) -> ModelInput:
        """Creates a model input into this model.

        Args:
            time_series: the time series to feed into this model
            group_name: the name of the input group on this model's connector to which to pass the data from this input
            output_index: which time dataset, out of the sequence of upstream time datasets, to pass to this model
            label_indexer: the slice of the upstream data to pass to this model

        Returns:
            the newly created input
        """
        return ModelInput.create(
            project=self.project,
            model=self.uuid,
            time_series=time_series,
            group_name=group_name,
            output_index=output_index,
            label_indexer=label_indexer,
        )

    def list_inputs(self) -> List[ModelInput]:
        """Lists all model inputs into this model."""
        return ModelInput.list(project=str(self.project), model=self.uuid)

    def create_fit_policy(
        self,
        schedule_timing: Union[Time, TimeDelta, ScheduleTiming],
        start_timing: TimeRangeBoundary,
        end_timing: TimeRangeBoundary,
        active: bool = True,
    ) -> ModelFitPolicy:
        return ModelFitPolicy.create(
            project=self.project,
            model=self.uuid,
            schedule_timing=schedule_timing,
            start_timing=start_timing,
            end_timing=end_timing,
            active=active,
        )

    def list_fit_policies(self) -> List[ModelFitPolicy]:
        return ModelFitPolicy.list(project=self.project, model=self.uuid)

    def list_fit_results(self) -> List[ModelFitResult]:
        return ModelFitResult.list(project=self.project, model=self.uuid)

    def get_fit_result(self, uuid: UUIDOrStr) -> ModelFitResult:
        return ModelFitResult.get(project=self.project, model=self.uuid, uuid=uuid)

    def list_run_results(self) -> List[ModelRunResultMetadata]:
        return ModelRunResult.list(project=self.project, model=self.uuid)

    def get_run_result(self, model_result: Union[ModelRunResultMetadata, UUIDOrStr]) -> ModelRunResult:
        return ModelRunResult.get(project=self.project, model=self.uuid, uuid=get_resource_uuid(model_result))

    def fit(self, start_timing: TimeRangeBoundary, end_timing: TimeRangeBoundary) -> ModelFitJob:
        """Create an ad hoc model fit job for this model."""
        model_fit_job = model_fit_job_create.request_sync(
            client=get_client(),
            project_uuid=str(self.project),
            uuid=str(self.uuid),
            json_body=NodeJobCreate(
                object="NodeJob",
                start_timing=to_timing_create(from_time_range_boundary(start_timing)),
                end_timing=to_timing_create(from_time_range_boundary(end_timing)),
            ),
        )

        return ModelFitJob.parse_obj(dict(model_fit_job.dict(), project=self.project))



from typing import List

from myst.models import base_model
from myst.openapi.models.hpo_get import HPOGet


class HPOGetList(base_model.BaseModel):
    """Schema for HPOGet list responses."""

    data: List[HPOGet]
    has_more: bool



from myst.client import Client
from myst.openapi.models.input_get import InputGet
from myst.openapi.models.input_update import InputUpdate


def request_sync(client: Client, project_uuid: str, model_uuid: str, uuid: str, json_body: InputUpdate) -> InputGet:
    """Updates an existing input for a model."""

    return client.request(
        method="patch",
        path=f"/projects/{project_uuid}/models/{model_uuid}/inputs/{uuid}",
        response_class=InputGet,
        request_model=json_body,
    )



from myst.client import Client
from myst.openapi.models.hpo_create import HPOCreate
from myst.openapi.models.hpo_get import HPOGet


def request_sync(client: Client, project_uuid: str, json_body: HPOCreate) -> HPOGet:
    """Creates an HPO in the project."""

    return client.request(
        method="post", path=f"/projects/{project_uuid}/hpos/", response_class=HPOGet, request_model=json_body
    )



from myst.client import Client
from myst.openapi.models.alias_get import AliasGet


def request_sync(client: Client, alias_uuid: str, project_uuid: str) -> AliasGet:
    """Update an alias to point to a different project."""

    return client.request(method="post", path=f"/alias/{alias_uuid}/project/{project_uuid}", response_class=AliasGet)



import enum
from typing import List
from uuid import UUID

from myst.connectors.operation_connector import OperationConnector


@enum.unique
class GroupName(str, enum.Enum):
    OPERANDS = "operands"


class GetDummies(OperationConnector):
    def __init__(self, categories: List[int]) -> None:
        super().__init__(uuid=UUID("5c1b5c26-7a76-4f95-a38b-9c92b3830a1b"), parameters=dict(categories=categories))



from typing import Any, Dict

from myst.client import Client
from myst.openapi.models.hpo_result_list import HPOResultList


def request_sync(client: Client, project_uuid: str, hpo_uuid: str, job_uuid: str) -> HPOResultList:
    """Filters for HPO results associated with a specific job."""

    params: Dict[str, Any] = {}
    params["job_uuid"] = job_uuid

    params = {k: v for k, v in params.items() if v is not None}

    return client.request(
        method="get",
        path=f"/projects/{project_uuid}/hpos/{hpo_uuid}/results/",
        response_class=HPOResultList,
        params=params,
    )



import enum
from typing import Optional, Union
from uuid import UUID

from myst.connectors.model_connector import ModelConnector


@enum.unique
class GroupName(str, enum.Enum):
    FEATURES = "features"
    TARGETS = "targets"
    SAMPLE_WEIGHTS = "sample_weights"


@enum.unique
class ExtraTreesRegressionCriterion(str, enum.Enum):
    MSE = "mse"
    MAE = "mae"


class ExtraTreesRegression(ModelConnector):
    def __init__(
        self,
        n_estimators: Optional[int] = None,
        criterion: Optional[ExtraTreesRegressionCriterion] = None,
        max_depth: Optional[int] = None,
        min_samples_split: Optional[Union[int, float]] = None,
        min_samples_leaf: Optional[Union[int, float]] = None,
        min_weight_fraction_leaf: Optional[float] = None,
        max_features: Optional[Union[int, float]] = None,
        max_leaf_nodes: Optional[int] = None,
        min_impurity_decrease: Optional[float] = None,
        bootstrap: Optional[bool] = None,
    ):
        super().__init__(
            uuid=UUID("0b42c668-90e3-4bf0-a2c7-7369e400b991"),
            parameters=dict(
                n_estimators=n_estimators,
                criterion=criterion,
                max_depth=max_depth,
                min_samples_split=min_samples_split,
                min_samples_leaf=min_samples_leaf,
                min_weight_fraction_leaf=min_weight_fraction_leaf,
                max_features=max_features,
                max_leaf_nodes=max_leaf_nodes,
                min_impurity_decrease=min_impurity_decrease,
                bootstrap=bootstrap,
            ),
        )



from myst.client import Client
from myst.openapi.models.time_series_create import TimeSeriesCreate
from myst.openapi.models.time_series_get import TimeSeriesGet


def request_sync(client: Client, project_uuid: str, json_body: TimeSeriesCreate) -> TimeSeriesGet:
    """Creates a time series."""

    return client.request(
        method="post",
        path=f"/projects/{project_uuid}/time_series/",
        response_class=TimeSeriesGet,
        request_model=json_body,
    )



from typing import Optional

from pydantic import Field
from typing_extensions import Literal

from myst.models import base_model
from myst.openapi.models.org_sharing_role import OrgSharingRole


class ProjectCreate(base_model.BaseModel):
    """Schema for project create requests."""

    title: str
    object_: Optional[Literal["Project"]] = Field("Project", alias="object")
    description: Optional[str] = None
    organization_sharing_enabled: Optional[bool] = False
    organization_sharing_role: Optional[OrgSharingRole] = None



from myst.client import Client
from myst.openapi.models.project_node_job_list import ProjectNodeJobList


def request_sync(client: Client, project_uuid: str) -> ProjectNodeJobList:
    """Lists jobs."""

    return client.request(
        method="get", path=f"/projects/{project_uuid}/nodes/-/jobs/", response_class=ProjectNodeJobList
    )



from myst.client import Client
from myst.openapi.models.operation_get import OperationGet


def request_sync(client: Client, project_uuid: str, uuid: str) -> OperationGet:
    """Gets an operation by its unique identifier."""

    return client.request(method="get", path=f"/projects/{project_uuid}/operations/{uuid}", response_class=OperationGet)



from typing import Optional

from pydantic import Field
from typing_extensions import Literal

from myst.models import base_model
from myst.openapi.models.backtest_result_get_metrics import BacktestResultGetMetrics


class BacktestResultGet(base_model.BaseModel):
    """Schema for model fit result get responses."""

    object_: Literal["BacktestResult"] = Field(..., alias="object")
    uuid: str
    create_time: str
    start_time: str
    end_time: str
    result_url: str
    metrics: Optional[BacktestResultGetMetrics] = None



import base64
import json
import os
import shutil
import tempfile
from pathlib import Path
from typing import TYPE_CHECKING, Any, Dict, List, Optional, Union
from uuid import UUID

import httpx
import pandas as pd

from myst.adapters.utils import get_resource_uuid
from myst.client import get_client
from myst.core.time.time import Time
from myst.models.base_model import BaseModel
from myst.models.time_dataset import TimeDataset
from myst.models.types import UUIDOrStr
from myst.openapi.api.projects.models.fit_results import get_model_fit_result, list_model_fit_results
from myst.openapi.api.projects.models.run_results import get_model_run_result, list_model_run_results
from myst.openapi.api.projects.time_series.run_results import get_time_series_run_result, list_time_series_run_results
from myst.resources.resource import Resource

if TYPE_CHECKING:  # Avoid circular imports.
    from myst.resources.model import Model
    from myst.resources.project import Project
    from myst.resources.time_series import TimeSeries


TENSORBOARD_FIT_STATE_KEY = "tensorboard_logs_base64"


class NodeResult(Resource):
    """Describes a result associated with a node.

    Attributes:
        project: the UUID of the project this result corresponds to
        node: the UUID of the node this result corresponds to
        start_time: the start time of this result
        end_time: the end time of this result
        as_of_time: the as of time of this result
    """

    project: UUID
    node: UUID
    start_time: Time
    end_time: Time
    as_of_time: Time


Inputs = Dict[str, List[TimeDataset]]


class InputsEnvelope(BaseModel):
    """The data that was used to calculate the fit result."""

    object: str
    inputs: Dict[str, List[TimeDataset]]


class ModelFitResult(NodeResult):
    """Results from a single run of a model fit.

    Attributes:
        inputs_url: an authenticated, time-expiring URL to the inputs used to create the fit result
        fit_state_url: an authenticated, time-expiring URL to the raw model fit state
    """

    inputs_url: Optional[str]
    fit_state_url: Optional[str]

    @classmethod
    def get(
        cls, project: Union["Project", UUIDOrStr], model: Union["Model", UUIDOrStr], uuid: UUIDOrStr
    ) -> "ModelFitResult":
        """Gets a specific fit result by its identifier."""
        model_fit_result = get_model_fit_result.request_sync(
            client=get_client(),
            project_uuid=str(get_resource_uuid(project)),
            model_uuid=str(get_resource_uuid(model)),
            uuid=str(uuid),
        )

        return ModelFitResult.parse_obj(dict(model_fit_result.dict(), project=get_resource_uuid(project)))

    @classmethod
    def list(cls, project: Union["Project", UUIDOrStr], model: Union["Model", UUIDOrStr]) -> List["ModelFitResult"]:
        """Lists all fit results for the model."""
        model_fit_results = list_model_fit_results.request_sync(
            client=get_client(), project_uuid=str(get_resource_uuid(project)), model_uuid=str(get_resource_uuid(model))
        )

        return [
            ModelFitResult.parse_obj(dict(model_fit_result.dict(), project=get_resource_uuid(project)))
            for model_fit_result in model_fit_results.data
        ]

    def _refresh(self) -> None:
        model_fit_result_detailed = get_model_fit_result.request_sync(
            client=get_client(), project_uuid=str(self.project), model_uuid=str(self.node), uuid=str(self.uuid)
        )
        self.inputs_url = model_fit_result_detailed.inputs_url
        self.fit_state_url = model_fit_result_detailed.fit_state_url

    def download_inputs(self) -> Dict[str, Any]:
        """Downloads the inputs that were used to generate the model fit result."""
        # Lazy load the inputs URL, for example if this object was acquired through a list rather than a get.
        if self.inputs_url is None:
            self._refresh()

        response = httpx.get(self.inputs_url)

        if response.status_code == 200:
            return InputsEnvelope.parse_raw(response.content).inputs
        else:
            raise RuntimeError("Could not download inputs.")

    def download_fit_state(self) -> Dict[str, Any]:
        """Downloads the fit state and parses it."""
        # Lazy load the fit state URL, for example if this object was acquired through a list rather than a get.
        if self.fit_state_url is None:
            self._refresh()

        response = httpx.get(self.fit_state_url)

        if response.status_code == 200:
            return json.loads(response.content)
        else:
            raise RuntimeError("Could not download fit state.")

    def download_tensorboard_logs(self, logs_dir: Union[Path, str] = "tensorboard_logs") -> None:
        """If available, downloads tensorboard logs for this model to the specified directory.

        Once you've downloaded tensorboard, you can view the logs with:

            $ tensorboard --logdir tensorboard_logs

        Note:
            - Log experiment names will be the model UUID
            - Log version names are formatted as "{fit start_time} – {fit end_time} ({fit as_of_time})"
            - This method supports downloading multiple tensorboard log experiment/versions to the same directory

        For example, if you run this method with `logs_dir` as "tensorboard_logs" on multiple model fit results across
        multiple models, you might see them show up in tensorboard with the organization:

            experiment / version
            -------------------------------
            model_uuid_1 / 2021-01-01T00:00:00Z – 2022-01-01T00:00:00Z (2022-06-01T00:00:00Z)
            model_uuid_1 / 2021-01-01T00:00:00Z – 2022-02-01T00:00:00Z (2022-06-01T00:00:00Z)
            model_uuid_2 / 2018-01-01T00:00:00Z – 2021-01-01T00:00:00Z (2022-06-02T00:00:00Z)

        Args:
            logs_dir: the top-level directory to download the logs to

        Raises:
            ValueError: if the fit state does not have associated tensorboard logs (key: "tensorboard_logs_base64")
        """
        fit_state = self.download_fit_state()
        if TENSORBOARD_FIT_STATE_KEY not in fit_state.keys():
            raise ValueError("This fit state does not contain tensorboard logs.")

        tensorboard_base64 = fit_state["tensorboard_logs_base64"]
        tensorboard_binary = base64.b64decode(tensorboard_base64)

        with tempfile.TemporaryDirectory() as tmp_dir_name:
            tmp_dir = Path(tmp_dir_name)
            tmp_file = tmp_dir / "my_file.tar"
            with tmp_file.open("wb") as f:
                f.write(tensorboard_binary)
            shutil.unpack_archive(filename=tmp_file, extract_dir=os.path.join(logs_dir, str(self.node)), format="tar")


class ModelRunResultMetadata(NodeResult):
    """Describes the metadata of a result produced by a model run.
    This representation does not contain the inputs and outputs of a model run.
    """


class TimeSeriesRunResultMetadata(NodeResult):
    """Describes the metadata of a result produced by a time series run.
    This representation does not contain the outputs of a time series run.
    """


class ModelRunResult(ModelRunResultMetadata):
    """Describes a result produced by a model run.
    Attributes:
        inputs: inputs to the model run
        outputs: outputs of the model run
    """

    inputs: Dict[str, List[TimeDataset]]
    outputs: List[TimeDataset]

    def _to_pandas_data_frame(self, group_name: str) -> pd.DataFrame:
        """Maps the inputs data for a specific `group_name` to a pandas data frame."""
        titles = [tds.metadata["node_title"] for tds in self.inputs[group_name] if tds.metadata is not None]
        return pd.concat(
            [tds.flatten().to_pandas_series() for tds in self.inputs[group_name]],
            keys=titles if titles else None,
            axis=1,
        )

    def to_pandas_data_frame(self, group_name: Optional[str] = None) -> pd.DataFrame:
        """Maps the inputs data for a specific `group_name` to a pandas data frame.

        Note that if `group_name` is not provided, the data frame columns will have multiple levels.

        Args:
            group_name: the specified group name.

        Returns:
            a pandas data frame for a specific `group_name` or a multilevel data frame if `group_name` not provided.
        """
        if group_name is not None:
            return self._to_pandas_data_frame(group_name)
        else:
            return pd.concat([self._to_pandas_data_frame(name) for name in [*self.inputs]], keys=[*self.inputs], axis=1)

    @classmethod
    def get(
        cls, project: Union["Project", UUIDOrStr], model: Union["Model", UUIDOrStr], uuid: UUIDOrStr
    ) -> "ModelRunResult":
        """Gets a specific run result by its identifier."""
        model_run_result = get_model_run_result.request_sync(
            client=get_client(),
            project_uuid=str(get_resource_uuid(project)),
            model_uuid=str(get_resource_uuid(model)),
            uuid=str(uuid),
        )

        return ModelRunResult.parse_obj(dict(model_run_result.dict(), project=get_resource_uuid(project)))

    @classmethod
    def list(
        cls, project: Union["Project", UUIDOrStr], model: Union["Model", UUIDOrStr]
    ) -> List[ModelRunResultMetadata]:
        """Lists all run results for the model."""
        model_run_results = list_model_run_results.request_sync(
            client=get_client(), project_uuid=str(get_resource_uuid(project)), model_uuid=str(get_resource_uuid(model))
        )

        return [
            ModelRunResultMetadata.parse_obj(dict(model_run_result.dict(), project=get_resource_uuid(project)))
            for model_run_result in model_run_results.data
        ]


class TimeSeriesRunResult(TimeSeriesRunResultMetadata):
    """Describes a result produced by a time series run.
    Attributes:
        outputs: outputs of the time series run
    """

    outputs: List[TimeDataset]

    @classmethod
    def get(
        cls, project: Union["Project", UUIDOrStr], time_series: Union["TimeSeries", UUIDOrStr], uuid: UUIDOrStr
    ) -> "TimeSeriesRunResult":
        """Gets a specific run result by its identifier."""
        time_series_run_result = get_time_series_run_result.request_sync(
            client=get_client(),
            project_uuid=str(get_resource_uuid(project)),
            time_series_uuid=str(get_resource_uuid(time_series)),
            uuid=str(uuid),
        )

        return TimeSeriesRunResult.parse_obj(dict(time_series_run_result.dict(), project=get_resource_uuid(project)))

    @classmethod
    def list(
        cls, project: Union["Project", UUIDOrStr], time_series: Union["TimeSeries", UUIDOrStr]
    ) -> List[TimeSeriesRunResultMetadata]:
        """Lists all run results for the time series."""
        time_series_run_results = list_time_series_run_results.request_sync(
            client=get_client(),
            project_uuid=str(get_resource_uuid(project)),
            time_series_uuid=str(get_resource_uuid(time_series)),
        )

        return [
            TimeSeriesRunResultMetadata.parse_obj(
                dict(time_series_run_result.dict(), project=get_resource_uuid(project))
            )
            for time_series_run_result in time_series_run_results.data
        ]



from typing import List, Union

from myst.models import base_model
from myst.openapi.models.model_fit_result_list_item import ModelFitResultListItem
from myst.openapi.models.model_run_result_list_item import ModelRunResultListItem
from myst.openapi.models.operation_run_result_get import OperationRunResultGet
from myst.openapi.models.source_run_result_get import SourceRunResultGet
from myst.openapi.models.time_series_insert_result_get import TimeSeriesInsertResultGet
from myst.openapi.models.time_series_run_result_list_item import TimeSeriesRunResultListItem


class ProjectNodeResultList(base_model.BaseModel):
    """Project result list schema."""

    data: List[
        Union[
            SourceRunResultGet,
            ModelRunResultListItem,
            OperationRunResultGet,
            TimeSeriesRunResultListItem,
            TimeSeriesInsertResultGet,
            ModelFitResultListItem,
        ]
    ]
    has_more: bool



from myst.client import Client
from myst.openapi.models.source_get import SourceGet


def request_sync(client: Client, project_uuid: str, uuid: str) -> SourceGet:
    """Deletes a source."""

    return client.request(method="delete", path=f"/projects/{project_uuid}/sources/{uuid}", response_class=SourceGet)
